#!/usr/bin/env python3

import gi
gi.require_version('Adw', '1')
gi.require_version('Gtk', '4.0')
gi.require_version('Gdk', '4.0')
gi.require_version('GdkPixbuf', '2.0')
gi.require_version('Poppler', '0.18')
gi.require_version('Pango', '1.0')
gi.require_version('GExiv2', '0.10')
from gi.repository import Gtk, Gio, GLib, Adw

from comipoli_window import ComipoliWindow
from comipoli_shortcuts import ComipoliShortcuts
from comipoli_preference import ComipoliPreference


MENU_XML = '''
<interface>
  <menu id='menubar'>
    <submenu>
      <attribute name='label' translatable='yes'>New Window</attribute>
      <item>
        <attribute name='label' translatable='yes'>test</attribute>
        <attribute name='action'>app.new_window_action</attribute>
      </item>
    </submenu>
  </menu>
</interface>
'''



class ComipoliApplication(Gtk.Application):
    def __init__(self, version):
        #GLib.set_prgname('Comipoli')
        self.version = version
        Gtk.Application.__init__(
            self,
            application_id='org.sasakima.Comipoli',
            flags=Gio.ApplicationFlags.HANDLES_COMMAND_LINE)
        # option
        self.add_main_option('version', b'v', GLib.OptionFlags.NONE, GLib.OptionArg.NONE, 'Show Comipoli Version', None)
        self.add_main_option('new-window', b'n', GLib.OptionFlags.NONE, GLib.OptionArg.NONE, 'New Comipoli Window', None)

    def do_command_line(self, command_line):
        # search option
        options = command_line.get_options_dict()
        if options.lookup_value('new-window', GLib.VariantType.new('b')):
            ComipoliWindow(self)
        # get filename
        arg = command_line.get_arguments()[1:]
        for s in arg:
            if not s.startswith('-'):
                if '//' in s:
                    f = Gio.file_new_for_uri(s)
                else:
                    f = Gio.file_new_for_path(s)
                if f.query_exists():
                    self.props.active_window.set_uri(f.get_uri())
                    break
        self.props.active_window.present() # gtk4
        return 0

    def do_handle_local_options(self, options):
        # get option
        if options.lookup_value('version', GLib.VariantType.new('b')):
            print(f'comipoli {self.version}')
            return 0
        return -1

    def do_shutdown(self):
        if GLib.file_test('/usr/share/glib-2.0/schemas/org.sasakima.Comipoli.gschema.xml', GLib.FileTest.EXISTS):
            settings = Gio.Settings(schema='org.sasakima.comipoli')
            settings.set_int('thumbnail-height', self.thumbnail_height)
            settings.set_boolean('esc-to-exit', self.esc_exit)
            settings.set_boolean('enable-pdf', self.is_pdf)
            xy = GLib.Variant('(ii)', (self.win_width, self.win_height))
            settings.set_value('window-size', xy)
        Gtk.Application.do_shutdown(self)

    def do_startup(self):
        Gtk.Application.do_startup(self)
        # Set Adwaita Style
        manager = Adw.StyleManager.get_default()
        manager.set_color_scheme(Adw.ColorScheme.FORCE_DARK)
        # Search Archiver
        self.is_unrar = GLib.spawn_command_line_sync('type unrar')[3] == 0
        self.is_7za = GLib.spawn_command_line_sync('type 7za')[3] == 0
        # Read GSettings
        if GLib.file_test('/usr/share/glib-2.0/schemas/org.sasakima.Comipoli.gschema.xml', GLib.FileTest.EXISTS):
            settings = Gio.Settings(schema='org.sasakima.comipoli')
            w, h = settings.get_value('window-size').unpack()
            self.win_width = w
            self.win_height = h
            self.left_to_right = settings.get_boolean('left-to-right')
            self.thumbnail_height = settings.get_int('thumbnail-height')
            self.esc_exit = settings.get_boolean('esc-to-exit')
            self.is_pdf = settings.get_boolean('enable-pdf')
        else:
            self.win_width = 640
            self.win_height = 480
            self.left_to_right = False
            self.thumbnail_height = 120
            self.esc_exit = True
            self.is_pdf = True
        # Application Menu
        builder = Gtk.Builder.new_from_string(MENU_XML, len(MENU_XML))
        appmenu = builder.get_object('menubar')
        self.set_menubar(appmenu)
        #
        # Accel @ app
        self.set_accels_for_action('app.new_file_action', ['<Control>O'])
        self.set_accels_for_action('app.new_window_action', ['<Control>N'])
        self.set_accels_for_action('app.shortcut_action', ['<Control>question'])
        self.set_accels_for_action('app.quit_action', ['<Control>Q'])
        # Acsel @ Win
        self.set_accels_for_action('win.action_esc', ['Escape'])
        self.set_accels_for_action('win.action_ctrl_w', ['<Control>W'])
        self.set_accels_for_action('win.action_space', ['space'])
        self.set_accels_for_action('win.action_backspace', ['BackSpace'])
        self.set_accels_for_action('win.action_ctrl_space', ['<Control>space'])
        self.set_accels_for_action('win.action_ctrl_backspace', ['<Control>BackSpace'])
        self.set_accels_for_action('win.action_left', ['Left'])
        self.set_accels_for_action('win.action_right', ['Right'])
        self.set_accels_for_action('win.action_ctrl_left', ['<Control>Left'])
        self.set_accels_for_action('win.action_ctrl_right', ['<Control>Right'])
        self.set_accels_for_action('win.action_home', ['Home'])
        self.set_accels_for_action('win.action_end', ['End'])
        self.set_accels_for_action('win.action_f8', ['F8'])
        self.set_accels_for_action('win.action_f9', ['F9'])
        self.set_accels_for_action('win.action_f10', ['F10'])
        self.set_accels_for_action('win.action_f11', ['F11'])
        # Action
        new_file_action   = Gio.SimpleAction(name='new_file_action')
        new_window_action = Gio.SimpleAction(name='new_window_action')
        preference_action = Gio.SimpleAction(name='preference_action')
        shortcut_action   = Gio.SimpleAction(name='shortcut_action')
        about_action      = Gio.SimpleAction(name='about_action')
        quit_action       = Gio.SimpleAction(name='quit_action')
        self.add_action(new_file_action)
        self.add_action(new_window_action)
        self.add_action(preference_action)
        self.add_action(shortcut_action)
        self.add_action(about_action)
        self.add_action(quit_action)
        # connect
        new_file_action.connect('activate', self.on_new_file_action)
        new_window_action.connect('activate', self.on_new_window_action)
        preference_action.connect('activate', self.on_preference_action)
        shortcut_action.connect('activate', self.on_shortcut_action)
        about_action.connect('activate', self.on_about_action)
        quit_action.connect('activate', self.on_quit_action)
        #
        Gtk.Window.set_default_icon_name('org.sasakima.Comipoli-symbolic')
        # Create Window
        ComipoliWindow(self)

    def on_new_file_action(self, action, parameter):
        self.props.active_window.open_dialog()

    def on_new_window_action(self, action, parameter):
        w = ComipoliWindow(self)
        w.present()

    def on_preference_action(self, action, parameter):
        dlg = ComipoliPreference(self)
        dlg.set_transient_for(self.props.active_window)
        #dlg.connect('response', self.on_preference_response)
        dlg.present()

    def on_shortcut_action(self, action, parameter):
        shortcuts = ComipoliShortcuts()
        self.props.active_window.set_help_overlay(shortcuts)
        shortcuts.set_visible(True)()

    def on_about_action(self, action, parameter):
        d = Adw.AboutWindow(
            developer_name='sasakima-nao',
            developers=['sasakima-nao'],
            #developers=['sasakima-nao <sasakimanao@gmail.com>'],
            #comments='Comic book Archive (CBZ|CBR|CB7) and PDF Viewer',
            copyright='© 2016-2023 sasakima-nao',
            license_type=Gtk.License.GPL_2_0,
            application_icon='org.sasakima.Comipoli',
            application_name='Comipoli',
            version=self.version,
            website='http://palepoli.skr.jp/',
            modal = True,
            transient_for=self.props.active_window)
        d.set_visible(True)()

    def on_quit_action(self, action, parameter):
        self.quit()

